<?php
/*  Copyright (c) 2018 - 2021 ArcLindon LLC, All Rights Reserved
 *  $Id: tmpltm_lib_templates.php 471 2021-09-03 11:04:31Z bill $
 *
 *  Changes:
 *  2021/09/03  bherndon    v13.0.4.1
 *  2021/09/02  bherndon    Removed empty select from arcl_web_dropdown
 *  2021/08/27  bherndon    PHP 8.0 compatibility deprecated function fixes
 *  2021/06/02  bherndon    'tmgpath' typo corrected in includes
 *  2018/05/19  bherndon    v12.0.0.0
 *  2018/05/15  bherndon    Drop-down list sorting
 *  2018/05/14  bherndon    Sorting db results on ordernum to preserve 
 *                          template variable order
 *  2018/05/13  bherndon    Updated function documentation
 *  2018/04/21  bherndon    Created from citem_lib_citation.php trunk@323
 */
 
// Include the admin library and logger
include_once($cms['tngpath'] . 'adminlib.php');
include_once($cms['tngpath'] . 'adminlog.php');
                        
// Include the settings
include($cms['tngpath'] . "$extspath/tmpltm_settings.php");

/*  Dropdown-list web-part
 *                        
 *  $name:      Name parameter of the <select> element
 *  $options:   List of items used for the <option> elements  
 *  $choose:    [optional] When specified, used to create an empty lead 
 *  $selected:  [optional] Selected identifier from the list of options
 *  $class:     [optional] Class parameter
 *  $event:     [optional] Event
 *  $action:    [optional] Event action (when event is specified)
 *              <option> element using the input
 *
 *  Returns:    String containing the <select> element
 */
function arcl_web_dropdown($name, array $options, $choose = null, 
                           $selected = null, $class = null, $event = null, 
                           $action = null)
{
  // start the select
  $etext = ($event) ? "$event=\"$action\"" : null;
  $part  = "<select name=\"$name\" id=\"$name\" $etext>\n";
    
  // generate the options
  if ($choose) {
    // generate the empty lead <option> element
    $part .= "<option style=\"font-style: italic;\" value=\"_none\">" 
             . "$choose</option>\n";
  }
  
  foreach($options as $key => $option) {
    $select = ($selected == $option) ? ' selected' : null;
    $part .= "<option value=\"$key\" $select>$option</option>\n";
  }

  // end the select
  $part .= '</select>'."\n";

  // return the web-part
  return $part;
}

/*  Given the name of a template, returns the path of the template's 
 *  configuration directory.  This may be the template's root directory 
 *  depending on the mod parameter values.
 *  
 *  $name:    A template name
 *  Returns:  A directory path as a string
 */
 function tmpltm_returnCfgPath($name) {
  global $rootpath, $tmpltm_dir_templates, $tmpltm_dir_config,
         $tmpltm_cfg_filename;
  
  return $rootpath . $tmpltm_dir_templates . '/' . $name 
         . (($tmpltm_dir_config != '') ? '/' . $tmpltm_dir_config : '');
}
 
/*  Given the name of a template, returns the path of the template's private 
 *  template config file (which may be in the template's root directory
 *  depending on the mod's parameters
 *  
 *  $name:    A template name
 *  Returns:  A directory path as a string
 */
function tmpltm_returnCfgFile($name) {
  global $tmpltm_cfg_filename;
  
  return tmpltm_returnCfgPath($name) . '/' . $tmpltm_cfg_filename;
}

function tmpltm_assembleKey($row) {
  $retval = $row['template'];
  $retval .= '_' . $row['keyname'];
  $retval .= ($row['language'] == "") ? "" : '_' . $row['language'];  
  
  return $retval;
}

/*  Returns an array of simple directory names suitable for using in a
 *  dropdown.  Template directory names are included if, and only if, there 
 *  exists a template configuration (default: '_templateconfig.php') file at 
 *  top-level of the directory or in a subdirectory identified by the 
 *  appropriate value in mod parameters.
 *  
 *  Returns:  An array of directory names
 */
function tmpltm_loadableTemplates() {
  global $rootpath, $tmpltm_dir_templates;
  
  $retval    = array();
  $tmpltpath = $rootpath . $tmpltm_dir_templates;
  
  if ($dir = @opendir($tmpltpath)) {
    while (($elem = readdir($dir)) !== false){
      if (is_dir("$rootpath$tmpltm_dir_templates/$elem")) {
        if (($elem != '.') and ($elem != '..')) {
          // we've got a directory; look for a template config file
          $cfgFile = tmpltm_returnCfgFile($elem);
          if (is_file($cfgFile)) {
            // we've got one; add the directory it to the array
            $retval[$elem] = $elem;
          }
        }
      }
    }
    closedir($dir);
  }
  
  // re-order the result
  ksort($retval);

  // return the array of templates that can be loaded
  return $retval;
}

/*  Returns an array of template names suitable for using in a
 *  dropdown.  Template names are assembed from the unique value of field
 *  'template' in the tng_templates table. Numeric values in this list are
 *  displayed with the term 'Template' prepended.
 *  
 *  Returns:  An array of template names
 */
function tmpltm_unloadableTemplates() {
  global $templates_table;
  
  // results
  $retval = array();
  $retcnt = 0;
  
  // a query
  $query = "SELECT DISTINCT template  
            FROM   $templates_table
            WHERE      template <> \"\"
                   AND template <> \"top\"";
  $rows  = tng_query($query);

  while ($row = tng_fetch_assoc($rows)) {
    // assemble key/value pairs
    $val = $row['template'];
    $retval[$val] = (is_numeric($val)) ? 'Template ' . $val : $val;
  }
  
  tng_free_result($rows);
  
  // re-order the return result
  ksort($retval);
               
  // return the key/value pairs representing templates
  return $retval;
}
  
/*  Given a name of a valid template, reads the data from that template's
 *  local configuration file (default: '_templateconfig.php').
 *  
 *  $name:    Template name corresponding to the template's directory under
 *            the TNG templates folder.
 *  Returns:  An associative array representing the templates key/value pairs
 *  Contract: The return value is null if the template doesn't exist or the 
 *            local template configuration file doesn't exist in the known
 *            location.
 *            
 */
function tmpltm_readTemplateData($name) {
  global $cms, $tmpltm_dir_templates, $tmpltm_dir_config,
         $tmpltm_cfg_filename;
  
  $retval = null;
  $path   = tmpltm_returnCfgFile($name);
  
  // fetch the template variables
  @include($path);
  
  // reporting
  if ($tmp and (count($tmp) > 0)) {
    adminwritelog("Template Manager: Read data from '$name'");
    $retval = $tmp;
  } else {
    adminwritelog("Template Manager: Failed to read template '$name' data using [$path].");
  }
  
  return $retval;
}  

/*  Given an associative array of key value pairs (typically derived from the 
 *  the TNG database, but could come from elsewhere) and the name of a template,
 *  writs the key/value pairs to the template config file identified by the 
 *  template name.
 *  
 *  $data:    Associative array of key/value pairs
 *  $name:    Template name whose configuration file should be written
 *  Returns:  The number of fields written to the file system when successful
 *            and 0 otherwise. 
 */
function tmpltm_writeTemplateData($data, $name) {

  $count  = 0;
  $path   = tmpltm_returnCfgFile($name);
  $file   = @fopen($path, "w");
  
  if ($file) {
    // found the template config file    
    $content = "<?php\n";
    
    // iterate over the data elements
    foreach ($data as $key => $value) {
      $value = addslashes($value);
      $value = str_replace("\\'","'",$value);
      $value = str_replace("\n","",$value);
      
      $content .= '$' . "tmp['$key'] = \"$value\";\n";
      $count++;
    }
    
    $content .= "?>\n";
    
    // write the content
    fwrite($file, $content);
    fclose($file);
    
    // report
    adminwritelog("Template Manager: Wrote $count fields to '$name'");
  } else {
    // report
    adminwritelog("Template Manager: Failed to write template '$name' data using [$path].");
  }

  return $count;
}

/*  Given the name of a valid template in the TNG database, returns an 
 *  associative array of key/value pairs whree the key is assembled from 
 *  tng_templates table's 'template', 'name', and 'language' fields.
 *  
 *  $name:    Template name, as stored in the TNG database
 *  Returns:  An associative array of key/value pairs.
 */
function tmpltm_readTemplateFields($name) {
  global $templates_table;
  
  // results
  $retval;
  $retcnt = 0;
  
  // a query
  $query = "SELECT * 
            FROM   $templates_table
            WHERE  template = \"$name\"
            ORDER BY ordernum";
            
  $rows  = tng_query($query);
  
  while ($row = tng_fetch_assoc($rows)) {
    // assemble key/value pairs
    $key = tmpltm_assembleKey($row);
    $retval[$key] = $row['value'];
    $retcnt++;
  }
  
  // report
  adminwritelog("Template Manager: Extracted $retcnt fields of '$name'");
  
  return $retval;
}

/*  Given a template's data in the form of key/value pairs, 'bursts' the key
 *  into it's individual fields and writes those to the TNG database 
 *  tng_templates table along with the values.
 *  
 *  $data:    Template keys/values (typically read from a template configuration
 *            file)
 *  Returns:  The number of rows written to the database.
 */
function tmpltm_writeTemplateFields($data) {
  global $templates_table;
    
  // set the initial order and return value
  $retval = 0;
  $torder = 0;
  
  // iterate over the data rows from the input
  foreach ($data as $key => $value) {
    // split the key and assign insert/update values
    $keyelems  = explode('_', $key);
    $tname     = $keyelems[0];
    $tkey      = $keyelems[1];
    $tvalue    = addslashes($value);
    
    // set the order
    $torder++;
    
    // set the language when present
    $tlanguage = (count($keyelems) > 2) ? $keyelems[2] : "";
    
    // construct the insert/update
    $ioru = "INSERT INTO $templates_table 
                         (template, ordernum, keyname, language, value)
             VALUES      (\"$tname\", \"$torder\", \"$tkey\", \"$tlanguage\", 
                          \"$tvalue\")
             ON DUPLICATE KEY UPDATE
                       template = \"$tname\",
                       ordernum = \"$torder\",
                       keyname  = \"$tkey\",
                       language = \"$tlanguage\",
                       value    = \"$tvalue\"";
    $result = tng_query($ioru);
    
    if ($result) $retval++;
  }
  
  // report
  adminwritelog("Template Manager: Committed $retval rows to the TNG database.");
  return $retval;
}

function tmpltm_wipeTemplateFields($name) {
  global $templates_table;
  
  // construct the insert/update
  $delq = "DELETE FROM $templates_table 
           WHERE       template = \"$name\"";

  $result = tng_query($delq);
  
  if ($result) 
    adminwritelog("Template Manager: Deleted $result rows of template '$name' from the database");
  else
    adminwritelog("Template Manager: Attempt to delete template '$name' failed.");

  return $result;
}

/*  Reads data from all templates resident in the TNG database and writes the
 *  data to the individual template configuration files in the file system.
 *  
 *  Returns:  The number of templates saved to the file system.
 */
function tmpltm_saveAll() {
  
  $retval  = 0;
  $stmplts = tmpltm_unloadableTemplates();
  
  foreach ($stmplts as $name => $value) {
    // read template fields from the database and then persist
    $tfields = tmpltm_readTemplateFields($name);
    $tname   = (is_numeric($name)) ? "template$name" : $name;
    $result  = tmpltm_writeTemplateData($tfields, $tname);
    if ($result > 0) {
      // we were successful - increment the count of saved templates
      adminwritelog("Template Manager: Saved '$name' to the TNG installation.");
      $retval++;
    }
  }
  
  // return the count
  return $retval;
}

/*  Reads data from all templates having a valid template configuraiton file
 *  and writes that data to the TNG database.
 *  
 *  Returns:  The number of templates read from the file system and written
 *            to the database.
 */
function tmpltm_loadAll() {
  
  $retval  = 0;
  $ltmplts = tmpltm_loadableTemplates();
  
  foreach ($ltmplts as $name => $value) {
    // load template from the filesystem then write to the database
    $tdata = tmpltm_readTemplateData($name);
    tmpltm_writeTemplateFields($tdata);
    adminwritelog("Template Manager: Loaded '$name' into the database.");
    $retval++;
  }
  
  // return the count
  return $retval;
}
?>
